c------------------------------------------------------------------------
c  program his2pdb    
c  version 4.23 (February 2000) re-edited
c
c  Authors: R.R.Gabdoulline, R.C.Wade
c  Copyright (c) 2000
c  European Molecular Biology Laboratory
c  Meyerhofstr. 1, Postfach 10.2209
c  D-69012, Heidelberg, Germany
c
c  Please send your contact address to get information on updates and
c  new features to "gabdoull@EMBL-Heidelberg.DE". Questions will be 
c  answered as soon as possible.
c
c  Please obtain a license before using this software from 
c  "wade@EMBL-Heidelberg.DE".  Conditions of usage are as specified 
c  in the license.
c
c  References:
c     R.R. Gabdoulline, R.C. Wade.     
c  Brownian dynamics simulation of protein-protein diffusional encounter.
c  (1998) Methods, 14, 329-341. 
c------------------------------------------------------------------------
c
			implicit real*8(a-h,o-z)
c.
c. converts sda history file (fort.55) to the set of pdb files
c. input is read from fort.5 (st in) and is of form:
c. 1st line: the name of 2nd protein pdb file
c. 2nd line: the name of sda history file
c. 3rd line: the first character (A) of pdb files to be created,
c.           (the files then have the names A####.pdb)
c. 
c. the first protein coordinates are the reference frame
c. the user should take care if the 2nd protein pdb file is
c. exactly the one used in brownian dynamics simulations !!
c. (no center shift)
c.
	  character*40 pdbf,hisf,outf
	  character*54 string
	  character*30 anames(3000)
	  character*30 anamet
	  character*4 atom
          character*5 nam
      dimension xv2(3,3000),xt2(3,3000),xt1(3,3000) 
      dimension xt2a(3,3000)
	  dimension xv(3),dum(3),xla(3)
	  dimension xc(3),xc1(3),xc2(3)
      dimension xp(3),yp(3),zp(3),xee(3),yee(3),zee(3)
      dimension vorig(3),ozz(3),zoz(3),zzo(3),vnew(3)
      dimension r(3),y1(3),y2(3)
	  data atom /'ATOM'/,outf/'Axxxx.pdb'/
c
      do k=1,3    
      ozz(k)=0.d0
      zoz(k)=0.d0
      zzo(k)=0.d0
      end do
       ozz(1)=1.d0
       zoz(2)=1.d0
       zzo(3)=1.d0
c
      call getarg(1,pdbf)
      call getarg(2,hisf)
      call getarg(3,outf)
c
      iupdb=57
	  ihisf=58
	  ioutf=59
          iouins=60
	  open(iupdb,file=pdbf,form='formatted',status='old')
	  open(ihisf,file=hisf,form='formatted',status='old')
	  read(ihisf,609) xc1,xc2
          xcdist=sqrt((xc1(1)-xc2(1))**2+
     +(xc1(2)-xc2(2))**2+(xc1(3)-xc2(3))**2)
          xla(1)=-2.6*xcdist
          xla(2)=2.0*xcdist
          xla(3)=0.0
 609  format('#',3f8.3)
c
c.. Read pdb file
      nat=0
 1    continue
	  read(iupdb,500,end=99) string
	  if(string(1:4).ne.atom) goto 1
	  nat=nat+1
	  write(anamet(1:30),500) string(1:30)
	  anames(nat)=anamet
	  read(string(1:54),501) xv
	  do m=1,3
	  xv2(m,nat)=xv(m)-xc2(m)
	  end do
	  goto 1
 99   continue
	  write(6,*) nat,'  atoms read from pdb file:'
	  write(6,500) pdbf
 500  format(a)
 501  format(30x,3f8.3)
 502  format(10f8.3)     
 503  format(10f8.3)     
c      	  
      write(6,*) 'Restoring first coordinate from BD history file'
c
c
	   iset=0
	   iwrit=0
      xc(1)=xc1(1)
      xc(2)=xc1(2)
      xc(3)=xc1(3)
	  write(6,'(8Hcenter: ,3f8.3)') xc
c
      read(ihisf,505,end=98) ntr,kcr,r,y1,y2,e1,e2,tim
        do k=1,3
        xp(k)=y1(k)
        yp(k)=y2(k)
        end do
        call cross(zp,xp,yp)
        call norm(zp)
       call tr(ozz,xee,xp,yp,zp)
       call tr(zoz,yee,xp,yp,zp)
       call tr(zzo,zee,xp,yp,zp)
       do i=1,nat
       do k=1,3
       vorig(k)=xv2(k,i)
       end do
       call tr(vorig,vnew,xee,yee,zee)
       do k=1,3
       xt2a(k,i)=(vnew(k)+r(k)+xc(k))
       end do
       end do
c.. write now...
       write(6,*) 'writing coordinate set #',iwrit
       write(6,503) r,y1,y2
	   iwrit=iwrit+1
	   open(ioutf,file=outf,form='formatted',status='new')
	   do iat=1,nat
	   write(ioutf,509) anames(iat),(xt2a(k,iat),k=1,3)
	   end do
       close(ioutf) 
c
 505   format(2i8,9f8.3,2e11.3,e16.7)
 508   format(3f8.3)
 509   format(a30,3f8.3)
 98    continue
	   call exit
	   end
c*************************************************************************/
      subroutine norm(wa)
			implicit real*8(a-h,o-z)
      dimension wa(3)
c       normalize:
      sum=wa(1)**2
      sum=sum+wa(2)**2
      sum=sum+wa(3)**2
      sum=sqrt(sum)
      wa(1)=wa(1)/sum
      wa(2)=wa(2)/sum
      wa(3)=wa(3)/sum
      return
      end
c*************************************************************************/
      subroutine cross(ti,a,b)
			implicit real*8(a-h,o-z)
      dimension ti(3),a(3),b(3)
      ti(1)=a(2)*b(3)-a(3)*b(2)
      ti(2)=a(3)*b(1)-a(1)*b(3)
      ti(3)=a(1)*b(2)-a(2)*b(1)
      return
      end
c*************************************************************************/
c* Transforms a vector from one coordinate system to another.		 */
c*  vorig = vector in original coordinate system			 */
c*  vnew  = resultant vector in new coordinate system			 */
c*  x,y,z = axis vectors of new coordinate system in terms of orig coord */
c*  syst.								 */
c*************************************************************************/
	subroutine tr(vorig,vnew,x,y,z)
			implicit real*8(a-h,o-z)
	dimension vorig(3),vnew(3),x(3),y(3),z(3)
c   1st make sure x,y,z are normalized:
	sumx=0.d0
	sumy=0.d0
	sumz=0.d0
	do k=1,3
	sumx=sumx+x(k)**2
	sumy=sumy+y(k)**2
	sumz=sumz+z(k)**2
	end do
	sumx=sqrt(sumx)
	sumy=sqrt(sumy)
	sumz=sqrt(sumz)
	do k=1,3
	x(k)=x(k)/sumx
	y(k)=y(k)/sumy
	z(k)=z(k)/sumz
	end do
c
c     now perform transformation:
c
	call dot(vnew(1),vorig,x)
	call dot(vnew(2),vorig,y)
	call dot(vnew(3),vorig,z)

	return
	end
c*************************************************************************/
      subroutine dot(sum,a,b)
			implicit real*8(a-h,o-z)
      dimension a(3),b(3)
      sum=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)
      return
      end
